unit DlgEditFontStyle;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, JvComponent, JvAppStorage,
  JvAppIniStorage, ImgList, VirtualTrees, AppEvnts, JvComponentBase;

type
  // m[hf[^
  PFontData = ^TFontData;
  TFontData = record
    Caption: string;
    Font: TFont;
  end;

  TEditFontStyleDlg = class(TForm)
    ButtonOK: TButton;
    ButtonCancel: TButton;
    Shape1: TShape;
    Label1: TLabel;
    IniStorage: TJvAppIniFileStorage;
    ImageList: TImageList;
    ButtonAdd: TButton;
    ButtonDelete: TButton;
    ButtonUp: TButton;
    ButtonDown: TButton;
    ButtonEdit: TButton;
    LabelPreview: TLabel;
    FontDialog: TFontDialog;
    TreeView: TVirtualStringTree;
    ApplicationEvents: TApplicationEvents;
    ButtonLoad: TButton;
    ButtonSave: TButton;
    SaveDialog: TSaveDialog;
    OpenDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure ButtonAddClick(Sender: TObject);
    procedure ButtonEditClick(Sender: TObject);
    procedure TreeViewGetNodeDataSize(Sender: TBaseVirtualTree;
      var NodeDataSize: Integer);
    procedure TreeViewGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure TreeViewNewText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; NewText: WideString);
    procedure TreeViewFreeNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure ButtonDeleteClick(Sender: TObject);
    procedure TreeViewFocusChanged(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure ButtonUpClick(Sender: TObject);
    procedure ButtonDownClick(Sender: TObject);
    procedure ButtonOKClick(Sender: TObject);
    procedure ButtonLoadClick(Sender: TObject);
    procedure ButtonSaveClick(Sender: TObject);
    procedure TreeViewDblClick(Sender: TObject);
    procedure TreeViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private 錾 }
    procedure ChangePreview;
    procedure InitNode(Node: PVirtualNode);
  public
    { Public 錾 }
  end;

var
  EditFontStyleDlg: TEditFontStyleDlg;

implementation

uses ModuleGlobal;

{$R *.dfm}


//----------------------------------------
// vr[ύX
//----------------------------------------
procedure TEditFontStyleDlg.ChangePreview;
var
  Data: PFontData;
begin
  if not Assigned(TreeView.FocusedNode) then
  begin
    LabelPreview.Visible := False;
    Exit;
  end;

  Data := TreeView.GetNodeData(TreeView.FocusedNode);
  LabelPreview.Visible := True;
  LabelPreview.Font.Assign(Data.Font);
end;


//----------------------------------------
// m[h
//----------------------------------------
procedure TEditFontStyleDlg.InitNode(Node: PVirtualNode);
var
  Data: PFontData;
begin
  Data := TreeView.GetNodeData(Node);
  Data.Font := TFont.Create;
end;


//----------------------------------------
// c[r[̃f[^TCY擾Cxg
//----------------------------------------
procedure TEditFontStyleDlg.TreeViewGetNodeDataSize(
  Sender: TBaseVirtualTree; var NodeDataSize: Integer);
begin
  NodeDataSize := SizeOf(TFontData);
end;


//----------------------------------------
// c[r[̃eLXg擾Cxg
//----------------------------------------
procedure TEditFontStyleDlg.TreeViewGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var
  Data: PFontData;
begin
  if TextType = ttNormal then
  begin
    Data := Sender.GetNodeData(Node);
    CellText := Data.Caption;
  end
  else
  begin
    CellText := '';
  end;
end;


//----------------------------------------
// c[r[̃eLXgύXCxg
//----------------------------------------
procedure TEditFontStyleDlg.TreeViewNewText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; NewText: WideString);
var
  Data: PFontData;
begin
  Data := Sender.GetNodeData(Node);
  Data.Caption := NewText;
end;


//----------------------------------------
// c[r[̃m[hjCxg
//----------------------------------------
procedure TEditFontStyleDlg.TreeViewFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  Data: PFontData;
begin
  Data := TreeView.GetNodeData(Node);
  Data.Caption := '';
  Data.Font.Free;
end;


//----------------------------------------
// tH[̏
//----------------------------------------
procedure TEditFontStyleDlg.FormCreate(Sender: TObject);
var
  I: Integer;
  Node: PVirtualNode;
  Data: PFontData;
  FStyle: TFontStyles;
  Fs :Byte absolute FStyle;
begin
  IniStorage.Reload;
  IniStorage.Path := 'UserFonts';

  for I := 0 to 15 do
  begin
    if IniStorage.ReadBoolean('UserFont' + IntToStr(I) + 'IsUse', False) then
    begin
      Node := TreeView.AddChild(nil);
      InitNode(Node);
      Data := TreeView.GetNodeData(Node);

      Data.Caption := IniStorage.ReadString('UserFont' + IntToStr(I) + 'StyleName', '');
      Data.Font.Name := IniStorage.ReadString('UserFont' + IntToStr(I) + 'Name', DEFAULT_FONT_NAME);
      Data.Font.Size := IniStorage.ReadInteger('UserFont' + IntToStr(I) + 'Size', DEFAULT_FONT_SIZE);
      Data.Font.Color := IniStorage.ReadInteger('UserFont' + IntToStr(I) + 'Color', DEFAULT_FONT_COLOR);
      Fs := IniStorage.ReadInteger('UserFont' + IntToStr(I) + 'Styles', 0);
      Data.Font.Style := FStyle;

    end;
  end;
  ChangePreview;
end;


//----------------------------------------
// uǉv{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonAddClick(Sender: TObject);
var
  Node: PVirtualNode;
  Data: PFontData;
  AFont: TFont;
begin
  Node := TreeView.AddChild(nil);

  AFont := TFont.Create;
  AFont.Name := DEFAULT_FONT_NAME;
  AFont.Size := DEFAULT_FONT_SIZE;
  AFont.Color := DEFAULT_FONT_COLOR;
  AFont.Style := [];

  Data := TreeView.GetNodeData(Node);
  Data.Caption := '';
  Data.Font := AFont;
  TreeView.ClearSelection;
  TreeView.Selected[Node] := True;
  TreeView.FocusedNode := Node;
  TreeView.EditNode(Node, -1);
end;


//----------------------------------------
// uҏWv{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonEditClick(Sender: TObject);
var
  Data: PFontData;
begin
  Data := TreeView.GetNodeData(TreeView.FocusedNode);
  FontDialog.Font.Assign(Data.Font);
  if FontDialog.Execute then
  begin
    Data.Font.Assign(FontDialog.Font);
    ChangePreview;
  end;
end;


//----------------------------------------
// u폜v{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonDeleteClick(Sender: TObject);
var
  Node: PVirtualNode;
begin
  Node := TreeView.FocusedNode.NextSibling;
  if Not Assigned(Node) then Node := TreeView.FocusedNode.PrevSibling;

  TreeView.DeleteNode(TreeView.FocusedNode);

  if Assigned(Node) then
  begin
    TreeView.FocusedNode := Node;
    TreeView.Selected[Node] := True;
  end;
end;


//----------------------------------------
// c[r[̃tH[JXύXCxg
//----------------------------------------
procedure TEditFontStyleDlg.TreeViewFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
begin
  ChangePreview;
end;


//----------------------------------------
// ACh
//----------------------------------------
procedure TEditFontStyleDlg.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
begin
  ButtonAdd.Enabled := (TreeView.RootNode.ChildCount < 16);

  if not Assigned(TreeView.FocusedNode) then
  begin
    ButtonEdit.Enabled := False;
    ButtonDelete.Enabled := False;
  end
  else
  begin
    ButtonEdit.Enabled := (TreeView.RootNode.ChildCount <> 0);
    ButtonDelete.Enabled := (TreeView.RootNode.ChildCount <> 0);
  end;

  if (TreeView.RootNode.ChildCount = 0) or (not Assigned(TreeView.FocusedNode)) then
  begin
    ButtonUp.Enabled := False;
    ButtonDown.Enabled := False;
  end
  else
  begin
    ButtonUp.Enabled := (TreeView.FocusedNode.Index <> 0);
    ButtonDown.Enabled := (TreeView.FocusedNode.Index <> TreeView.RootNode.ChildCount-1);
  end;
end;


//----------------------------------------
// uցv{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonUpClick(Sender: TObject);
begin
  TreeView.MoveTo(TreeView.FocusedNode, TreeView.FocusedNode.PrevSibling, amInsertBefore, False);
end;


//----------------------------------------
// uցv{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonDownClick(Sender: TObject);
begin
  TreeView.MoveTo(TreeView.FocusedNode, TreeView.FocusedNode.NextSibling, amInsertAfter, False);
end;


//----------------------------------------
// uOKv{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonOKClick(Sender: TObject);
var
  I: Cardinal;
  Node: PVirtualNode;
  Data: PFontData;
  FStyle: TFontStyles;
  Fs :Byte absolute FStyle;
begin
  IniStorage.Path := 'UserFonts';

  Node := TreeView.RootNode;
  Node := Node.FirstChild;
  for I := 0 to 15 do
  begin
    if not Assigned(Node) then
    begin
      IniStorage.WriteBoolean('UserFont' + IntToStr(I) + 'IsUse', False);
    end
    else
    begin
      IniStorage.WriteBoolean('UserFont' + IntToStr(I) + 'IsUse', True);
      Data := TreeView.GetNodeData(Node);

      IniStorage.WriteString('UserFont' + IntToStr(I) + 'StyleName', Data.Caption);
      IniStorage.WriteString('UserFont' + IntToStr(I) + 'Name', Data.Font.Name);
      IniStorage.WriteInteger('UserFont' + IntToStr(I) + 'Size', Data.Font.Size);
      IniStorage.WriteInteger('UserFont' + IntToStr(I) + 'Color', Data.Font.Color);
      FStyle := Data.Font.Style;
      IniStorage.WriteInteger('UserFont' + IntToStr(I) + 'Styles', Fs);
      Node := Node.NextSibling;
    end;
  end;

  IniStorage.Flush;
end;


//----------------------------------------
// uǂݍ݁v{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonLoadClick(Sender: TObject);
var
  SL: TStringList;
  Count: Integer;
  I: Integer;
  Line: Integer;
  Node: PVirtualNode;
  ChildNode: PVirtualNode;
  Data: PFontData;
  FStyle: TFontStyles;
  Fs :Byte absolute FStyle;
begin
  if not OpenDialog.Execute then Exit;

  SL := TStringList.Create;
  try
    TreeView.Clear;
    Node := TreeView.RootNode;

    SL.LoadFromFile(OpenDialog.FileName);
    Count := StrToInt(SL[0]);
    Line := 1;
    for I := 0 to Count -1 do
    begin
      ChildNode := TreeView.AddChild(Node);
      InitNode(ChildNode);
      Data := TreeView.GetNodeData(ChildNode);
      Data.Caption := SL[Line];
      Inc(Line);
      Data.Font.Name := SL[Line];
      Inc(Line);
      Data.Font.Size := StrToInt(SL[Line]);
      Inc(Line);
      Data.Font.Color := StrToInt(SL[Line]);
      Inc(Line);
      Fs := StrToInt(SL[Line]);
      Data.Font.Style := FStyle;
      Inc(Line);
    end;

  finally
    SL.Free;
  end;
end;


//----------------------------------------
// uۑv{^NbN
//----------------------------------------
procedure TEditFontStyleDlg.ButtonSaveClick(Sender: TObject);
var
  SL: TStringList;
  Node: PVirtualNode;
  Data: PFontData;
  FStyle: TFontStyles;
  Fs :Byte absolute FStyle;
begin
  if not SaveDialog.Execute then Exit;

  SL := TStringList.Create;
  try
    Node := TreeView.RootNode;
    SL.Add(IntToStr(Node.ChildCount));
    Node := Node.FirstChild;

    while Assigned(Node) do
    begin
      Data := TreeView.GetNodeData(Node);

      SL.Add(Data.Caption);
      SL.Add(Data.Font.Name);
      SL.Add(IntToStr(Data.Font.Size));
      SL.Add(IntToStr(Data.Font.Color));
      FStyle := Data.Font.Style;
      SL.Add(IntToStr(Fs));

      Node := Node.NextSibling;
    end;
    SL.SaveToFile(SaveDialog.FileName);
  finally
    SL.Free;
  end;
end;


//----------------------------------------
// c[_uNbN
//----------------------------------------
procedure TEditFontStyleDlg.TreeViewDblClick(Sender: TObject);
begin
  if Assigned(TreeView.FocusedNode) then ButtonEditClick(Sender);
end;

procedure TEditFontStyleDlg.TreeViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
  begin
    TreeViewDblClick(Sender);
  end;
end;


end.
