unit FormEditor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, JvComponentBase, JvEmbeddedForms, StdCtrls, JvExStdCtrls,
  RichEdit, JvRichEdit, NanaRichEdit, ExtCtrls, Ruler, ExtDlgs,
  ActnList, ImgList;

type
  TEditorForm = class(TForm)
    JvEmbeddedFormLink: TJvEmbeddedFormLink;
    Ruler: TRuler;
    Editor: TNanaRichEdit;
    LeftM: TPanel;
    EditorActionList: TActionList;
    ActionSetEditRect: TAction;
    ActionBold: TAction;
    ActionItalic: TAction;
    ActionUnderline: TAction;
    ActionStrikeOut: TAction;
    ActionAlignLeft: TAction;
    ActionAlignRight: TAction;
    ActionAlignCenter: TAction;
    ActionBullets: TAction;
    ActionNumbering: TAction;
    ActionSuperScript: TAction;
    ActionSubScript: TAction;
    ActionFontDlg: TAction;
    ActionInsertLink: TAction;
    ActionInsertPicture: TAction;
    ActionInsertObject: TAction;
    ActionInsertTable: TAction;
    ActionEditCopy: TAction;
    ActionEditCut: TAction;
    ActionEditPaste: TAction;
    ActionEditFormatPaste: TAction;
    ActionEditUndo: TAction;
    ActionEditRedo: TAction;
    ActionEditSelectAll: TAction;
    OpenPictureDialog: TOpenPictureDialog;
    FontDialog: TFontDialog;
    ImageList: TImageList;
    procedure ActionSetEditRectExecute(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure EditorResizeRequest(Sender: TObject; Rect: TRect);
    procedure EditorSelectionChange(Sender: TObject);
    procedure RulerMarginChanged(Sender: TObject);
    procedure RulerIndentChanged(Sender: TObject);
    procedure RulerRulerItemSelect(Sender: TObject; X: Integer);
    procedure RulerRulerItemMove(Sender: TObject; X: Integer;
      Removing: Boolean);
    procedure RulerRulerItemRelease(Sender: TObject);
    procedure RulerTabChanged(Sender: TObject);
    procedure ActionBoldExecute(Sender: TObject);
    procedure ActionBoldUpdate(Sender: TObject);
    procedure ActionItalicExecute(Sender: TObject);
    procedure ActionItalicUpdate(Sender: TObject);
    procedure ActionUnderlineExecute(Sender: TObject);
    procedure ActionUnderlineUpdate(Sender: TObject);
    procedure ActionStrikeOutExecute(Sender: TObject);
    procedure ActionStrikeOutUpdate(Sender: TObject);
    procedure ActionAlignLeftExecute(Sender: TObject);
    procedure ActionAlignLeftUpdate(Sender: TObject);
    procedure ActionAlignRightExecute(Sender: TObject);
    procedure ActionAlignRightUpdate(Sender: TObject);
    procedure ActionAlignCenterExecute(Sender: TObject);
    procedure ActionAlignCenterUpdate(Sender: TObject);
    procedure ActionBulletsExecute(Sender: TObject);
    procedure ActionBulletsUpdate(Sender: TObject);
    procedure ActionNumberingExecute(Sender: TObject);
    procedure ActionNumberingUpdate(Sender: TObject);
    procedure ActionSuperScriptExecute(Sender: TObject);
    procedure ActionSuperScriptUpdate(Sender: TObject);
    procedure ActionSubScriptExecute(Sender: TObject);
    procedure ActionSubScriptUpdate(Sender: TObject);
    procedure ActionInsertLinkExecute(Sender: TObject);
    procedure ActionInsertPictureExecute(Sender: TObject);
    procedure ActionInsertObjectExecute(Sender: TObject);
    procedure ActionInsertTableExecute(Sender: TObject);
    procedure ActionFontDlgExecute(Sender: TObject);
    procedure ActionEditCopyExecute(Sender: TObject);
    procedure ActionEditCopyUpdate(Sender: TObject);
    procedure ActionEditCutExecute(Sender: TObject);
    procedure ActionEditCutUpdate(Sender: TObject);
    procedure ActionEditPasteExecute(Sender: TObject);
    procedure ActionEditPasteUpdate(Sender: TObject);
    procedure ActionEditFormatPasteExecute(Sender: TObject);
    procedure ActionEditFormatPasteUpdate(Sender: TObject);
    procedure ActionEditUndoExecute(Sender: TObject);
    procedure ActionEditUndoUpdate(Sender: TObject);
    procedure ActionEditRedoExecute(Sender: TObject);
    procedure ActionEditRedoUpdate(Sender: TObject);
    procedure ActionEditSelectAllExecute(Sender: TObject);
    procedure EditorMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure EditorDblClick(Sender: TObject);
  private
    { Private 錾 }
    FUpdating: Boolean;
    FSaveX: Integer;
    FControlCanvas: TControlCanvas;

    procedure DrawRulerMarkLine;
    procedure SetFontStyle(Style: TFontStyle);
  public
    { Public 錾 }
    function  IsNodeLink(Editor: TJvRichEdit; Index: Integer;
      var S: String): Boolean; overload;
    function  IsNodeLink(Pt: TPoint; var S: String): Boolean; overload;
  end;

var
  EditorForm: TEditorForm;

implementation

uses DlgInsertTable, FormMain;

{$R *.dfm}


//----------------------------------------
// w肵ʒuɃm[hN邩ǂׂ
// (m[hN̕255ɐĂ܂)
//----------------------------------------
function TEditorForm.IsNodeLink(Editor: TJvRichEdit; Index: Integer;
  var S: String): Boolean;
var
  PrevEnd: Integer;
  NowStart: Integer;
  NowEnd: Integer;
  I: Integer;
begin
  Result := False;

  PrevEnd := Editor.FindText('</nlink>', Index, 255, [stBackward]);
  NowStart := Editor.FindText('<nlink>', Index, 255, [stBackward]);
  if NowStart > PrevEnd then
  begin
    NowEnd := Editor.FindText('</nlink>', NowStart, 100, []);
    if NowEnd <> -1 then
    begin
      NowStart := NowStart + Length('<nlink>');
      for I := 0 to NowEnd -1 - NowStart do
      begin
        S := S + Editor.GetTextRange(NowStart + I, NowStart + I + 1);
      end;
      Result := True;
    end;
  end;
end;


//----------------------------------------
// }EẌʒuɃm[hN邩ǂׂ
//----------------------------------------
function TEditorForm.IsNodeLink(Pt: TPoint; var S: String): Boolean;
var
  Index: Integer;
begin
  Result := False;

  Index := Editor.Perform(Messages.EM_CHARFROMPOS, 0, Integer(@Pt));

  if Index < 0 then Exit;

  Result := IsNodeLink(Editor, Index, S);
end;


//----------------------------------------
// GfB^̋`͈͂ݒ
//----------------------------------------
procedure TEditorForm.ActionSetEditRectExecute(Sender: TObject);
var
  R: TRect;
  M: Integer;
begin
  if not Assigned(Editor) then Exit;
  if not Assigned(LeftM) then Exit;

  if Editor.WordWrap then
  begin
    R := Rect(
      Round(Ruler.UnitsToPixs(Ruler.LeftMargin)),
      0,
      //Round(Ruler.UnitsToPixs(Ruler.PageWidth)),
      Editor.ClientWidth,
      Editor.ClientHeight
    );
  end
  else
  begin
    R := Rect(
      Round(Ruler.UnitsToPixs(Ruler.LeftMargin)),
      0,
      Editor.ClientWidth,
      Editor.ClientHeight
    );
  end;

  if Win32Platform = VER_PLATFORM_WIN32_NT then InflateRect(R, - 1, 0);

  SendMessage(Editor.Handle, EM_SETRECT, 0, LongInt(@R));

  if Win32Platform = VER_PLATFORM_WIN32_NT then InflateRect(R, + 1, 0);

  M := 0;
  if Win32Platform = VER_PLATFORM_WIN32_NT then M := 1;

  // }[Wʒu킹
  LeftM.Top := Editor.Top + 2;
  LeftM.Height := Editor.ClientHeight;
  LeftM.Left := Editor.Left + R.Left -1 + M;
end;


//----------------------------------------
// TCY
//----------------------------------------
procedure TEditorForm.FormResize(Sender: TObject);
begin
  ActionSetEditRectExecute(Sender);
end;


//----------------------------------------
// GfB^TCYύXv
//----------------------------------------
procedure TEditorForm.EditorResizeRequest(Sender: TObject; Rect: TRect);
begin
  ActionSetEditRectExecute(Sender);
end;


//----------------------------------------
// I͈͂ύXꂽ
//----------------------------------------
procedure TEditorForm.EditorSelectionChange(Sender: TObject);
var
  I: Integer;
  M: Extended;
begin
  FUpdating := True;
  try
    M := 1 / Ruler.MultiPoints;
    Ruler.FirstIndent := M * Editor.Paragraph.FirstIndent;
    Ruler.LeftIndent  := M * Editor.Paragraph.LeftIndent;
    Ruler.RightIndent := M * Editor.Paragraph.RightIndent;

    Ruler.Tabs.Clear;
    for I := 0 to Editor.Paragraph.TabCount - 1 do
    begin
      Ruler.Tabs.Add.Points := Editor.Paragraph.Tab[I];
    end;

    // CtH[ɈˑȂA
    // CtH[Ńc[̃CxgLb`̂ʓ|
    // CtH[̃c[ύXĂ
    MainForm.EditorSelectionChange(Sender);
  finally
    FUpdating := False;
  end;
end;


//----------------------------------------
// }[WύX
//----------------------------------------
procedure TEditorForm.RulerMarginChanged(Sender: TObject);
begin
  ActionSetEditRectExecute(Sender);
end;


//----------------------------------------
// CfgύX
//----------------------------------------
procedure TEditorForm.RulerIndentChanged(Sender: TObject);
var
  M: Extended;
begin
  if not Assigned(Editor) then Exit;
  if FUpdating then Exit;

  Editor.Invalidate;
  M := Ruler.MultiPoints;
  Editor.Paragraph.FirstIndent := Round(M * Ruler.FirstIndent);
  Editor.Paragraph.LeftIndent  := Round(M * Ruler.LeftIndent);
  Editor.Paragraph.RightIndent := Round(M * Ruler.RightIndent);
  EditorSelectionChange(Sender);
end;


//----------------------------------------
// [ړ̓_`
//----------------------------------------
procedure TEditorForm.DrawRulerMarkLine;
begin
  if Assigned(FControlCanvas) then
  begin
    if FSaveX >= 0 then
    begin
      FControlCanvas.MoveTo(FSaveX, 0);
      FControlCanvas.LineTo(FSaveX, Editor.Height);
    end;
  end;
end;


//----------------------------------------
// [ړJn
//----------------------------------------
procedure TEditorForm.RulerRulerItemSelect(Sender: TObject; X: Integer);
begin
  Editor.Update;
  if not Assigned(FControlCanvas) then FControlCanvas := TControlCanvas.Create;

  FControlCanvas.Control := Editor;
  FControlCanvas.Pen.Color := clBlack;
  FControlCanvas.Pen.Mode := pmXor;
  FControlCanvas.Pen.Style := psDot;

  FSaveX := X - Ruler.Inset;
  DrawRulerMarkLine;
end;


//----------------------------------------
// [ړ
//----------------------------------------
procedure TEditorForm.RulerRulerItemMove(Sender: TObject; X: Integer;
  Removing: Boolean);
begin
  DrawRulerMarkLine;
  if Removing then
  begin
    FSaveX := -1;
  end
  else
  begin
    FSaveX := X - Ruler.Inset;
  end;

  DrawRulerMarkLine;
end;


//----------------------------------------
// [ړI
//----------------------------------------
procedure TEditorForm.RulerRulerItemRelease(Sender: TObject);
begin
  DrawRulerMarkLine;
  FreeAndNil(FControlCanvas);
end;


//----------------------------------------
// ^uύX(^uA^u)
//----------------------------------------
procedure TEditorForm.RulerTabChanged(Sender: TObject);
var
  I: Integer;
begin
  if FUpdating then Exit;

  Editor.Paragraph.TabCount := Ruler.Tabs.Count + 1;
  for I := 0 to Ruler.Tabs.Count - 1 do
  begin
    Editor.Paragraph.Tab[I] := Round(Ruler.Tabs[I].Points);
  end;
  Editor.Paragraph.TabCount := Ruler.Tabs.Count;
end;


//----------------------------------------
// tHgX^CύX
//----------------------------------------
procedure TEditorForm.SetFontStyle(Style: TFontStyle);
begin
  if FUpdating then Exit;

  if Style in Editor.SelAttributes.Style then
  begin
    Editor.SelAttributes.Style := Editor.SelAttributes.Style - [Style];
  end
  else
  begin
    Editor.SelAttributes.Style := Editor.SelAttributes.Style + [Style];
  end;
end;


//----------------------------------------
// 
//----------------------------------------
procedure TEditorForm.ActionBoldExecute(Sender: TObject);
begin
  SetFontStyle(fsBold);
end;

procedure TEditorForm.ActionBoldUpdate(Sender: TObject);
begin
  ActionBold.Checked := fsBold in Editor.SelAttributes.Style;
end;


//----------------------------------------
// C^bN
//----------------------------------------
procedure TEditorForm.ActionItalicExecute(Sender: TObject);
begin
  SetFontStyle(fsItalic);
end;

procedure TEditorForm.ActionItalicUpdate(Sender: TObject);
begin
  ActionItalic.Checked := fsItalic in Editor.SelAttributes.Style;
end;


//----------------------------------------
// 
//----------------------------------------
procedure TEditorForm.ActionUnderlineExecute(Sender: TObject);
begin
  SetFontStyle(fsUnderline);
end;

procedure TEditorForm.ActionUnderlineUpdate(Sender: TObject);
begin
  ActionUnderline.Checked := fsUnderline in Editor.SelAttributes.Style;
end;


//----------------------------------------
// 
//----------------------------------------
procedure TEditorForm.ActionStrikeOutExecute(Sender: TObject);
begin
  SetFontStyle(fsStrikeOut);
end;

procedure TEditorForm.ActionStrikeOutUpdate(Sender: TObject);
begin
  ActionStrikeOut.Checked := fsStrikeOut in Editor.SelAttributes.Style;
end;


//----------------------------------------
// 
//----------------------------------------
procedure TEditorForm.ActionAlignLeftExecute(Sender: TObject);
begin
  if FUpdating then Exit;

  Editor.Paragraph.Alignment := paLeftJustify;
end;

procedure TEditorForm.ActionAlignLeftUpdate(Sender: TObject);
begin
  ActionAlignLeft.Checked := (paLeftJustify = Editor.Paragraph.Alignment);
end;


//----------------------------------------
// E
//----------------------------------------
procedure TEditorForm.ActionAlignRightExecute(Sender: TObject);
begin
  if FUpdating then Exit;

  Editor.Paragraph.Alignment := paRightJustify;
end;

procedure TEditorForm.ActionAlignRightUpdate(Sender: TObject);
begin
  ActionAlignRight.Checked := (paRightJustify = Editor.Paragraph.Alignment);
end;


//----------------------------------------
// 
//----------------------------------------
procedure TEditorForm.ActionAlignCenterExecute(Sender: TObject);
begin
  if FUpdating then Exit;

  Editor.Paragraph.Alignment := paCenter;
end;

procedure TEditorForm.ActionAlignCenterUpdate(Sender: TObject);
begin
  ActionAlignCenter.Checked := (paCenter = Editor.Paragraph.Alignment);
end;


//----------------------------------------
// ӏ
//----------------------------------------
procedure TEditorForm.ActionBulletsExecute(Sender: TObject);
begin
  if FUpdating then Exit;

  if Editor.Paragraph.Numbering = nsBullet then
  begin
    Editor.Paragraph.Numbering := nsNone;
  end
  else
  begin
    Editor.Paragraph.Numbering := nsBullet;
  end;
end;

procedure TEditorForm.ActionBulletsUpdate(Sender: TObject);
begin
  ActionBullets.Checked := (Editor.Paragraph.Numbering = nsBullet);
end;


//----------------------------------------
// ԍt
//----------------------------------------
procedure TEditorForm.ActionNumberingExecute(Sender: TObject);
begin
  if FUpdating then Exit;

  if Editor.Paragraph.Numbering = nsArabicNumbers then
  begin
    Editor.Paragraph.Numbering := nsNone;
  end
  else
  begin
    Editor.Paragraph.NumberingStart := 1;
    Editor.Paragraph.Numbering := nsArabicNumbers;
  end;
end;

procedure TEditorForm.ActionNumberingUpdate(Sender: TObject);
begin
  ActionNumbering.Checked := (Editor.Paragraph.Numbering = nsArabicNumbers);
end;


//----------------------------------------
// t
//----------------------------------------
procedure TEditorForm.ActionSuperScriptExecute(Sender: TObject);
begin
  if FUpdating then Exit;

  if ssSuperscript = Editor.SelAttributes.SubscriptStyle then
  begin
    Editor.SelAttributes.SubscriptStyle := ssNone;
  end
  else
  begin
    Editor.SelAttributes.SubscriptStyle := ssSuperScript;
  end;
end;

procedure TEditorForm.ActionSuperScriptUpdate(Sender: TObject);
begin
  ActionSuperScript.Checked := (ssSuperScript = Editor.SelAttributes.SubscriptStyle);
end;


//----------------------------------------
// t
//----------------------------------------
procedure TEditorForm.ActionSubScriptExecute(Sender: TObject);
begin
  if FUpdating then Exit;

  if ssSubscript = Editor.SelAttributes.SubscriptStyle then
  begin
    Editor.SelAttributes.SubscriptStyle := ssNone;
  end
  else
  begin
    Editor.SelAttributes.SubscriptStyle := ssSubscript;
  end;
end;

procedure TEditorForm.ActionSubScriptUpdate(Sender: TObject);
begin
  ActionSubScript.Checked := (ssSubScript = Editor.SelAttributes.SubscriptStyle);
end;


//----------------------------------------
// N}
//----------------------------------------
procedure TEditorForm.ActionInsertLinkExecute(Sender: TObject);
var
  Idx: Integer;
  L: Integer;
  F: TFont;
begin
  Idx := Editor.SelStart;
  L := Editor.SelLength;
  Editor.InsertFormatText(Idx + L, '</nlink>');
  Editor.InsertFormatText(Idx, '<nlink>');
  Editor.SelStart := Idx + Length('<nlink>');
  Editor.SelLength := L;

  // NX^Cɐݒ
  F := TFont.Create;
  try
    F.Name := Editor.SelAttributes.Name;
    F.Size := Editor.SelAttributes.Size;
    F.Color := clBlue;
    F.Style := [fsUnderline];
    Editor.SelAttributes.Assign(F);
  finally
    F.Free;
  end;
end;


//----------------------------------------
// 摜}
//----------------------------------------
procedure TEditorForm.ActionInsertPictureExecute(Sender: TObject);
var
  Pic: TPicture;
begin
  if OpenPictureDialog.Execute then
  begin
    Pic := TPicture.Create;
    try
      Pic.LoadFromFile(OpenPictureDialog.FileName);
      Editor.InsertGraphic(Pic.Graphic, True);
    finally
      Pic.Free;
    end;
  end;
end;


//----------------------------------------
// OLEIuWFNg}
//----------------------------------------
procedure TEditorForm.ActionInsertObjectExecute(Sender: TObject);
begin
  Editor.InsertObjectDialog;
end;


//----------------------------------------
// \}
//----------------------------------------
procedure TEditorForm.ActionInsertTableExecute(Sender: TObject);
const
  // b`eLXg^O
  Head =
    '{\rtf1';
//    '{\rtf1' +
//    '\ansi\ansicpg1252\deff0\deflang1033{\fonttbl' +
//    '{\f0\froman\fprq2\fcharset0 Times New Roman;}' +
//    '{\f1\fswiss\fcharset0 Arial;}}' +
//    '{\*\generator Msftedit 5.41.15.1503;}\viewkind4\uc1';

  Row =
    '\trowd\trgaph108\trleft8' +
    '\trbrdrl\brdrs\brdrw10 \trbrdrt' +
    '\brdrs\brdrw10 \trbrdrr\brdrs' +
    '\brdrw10 \trbrdrb\brdrs\brdrw10' +
    '\trpaddl108\trpaddr108\trpaddfl3\trpaddfr3';

  Col =
    '\clbrdrl\brdrw10\brdrs\clbrdrt\brdrw10\brdrs\clbrdrr' +
    '\brdrw10\brdrs\clbrdrb\brdrw10\brdrs\cellx';

  Endcell = '\cell';

  Endrow = '\row';
var
  Dlg: TInsertTableDlg;
  Rows: Integer;
  Cols: Integer;
  TableW: Integer;
  W: Integer;
  SL: TStringList;
  I: Integer;
  J: Integer;
  ES: TEditStream;
  Rich: TJvRichEdit;
  MS: TMemoryStream;

  // R[obN֐
  function myCallback(dwCookie: DWORD; pbBuff: PByte;
    cb: Longint; var pcb: Longint): DWORD; stdcall;
  var
    MS: TMemoryStream;
  begin
    Result := 0;
    MS := TMemoryStream(dwCookie);
    pcb := MS.Read(pbBuff^, cb);
  end;

begin
  Dlg := TInsertTableDlg.Create(Self);
  try
    if mrOK <> Dlg.ShowModal then Exit;

    Rows := StrToInt(Dlg.SpinEditRow.Text);
    Cols := StrToInt(Dlg.SpinEditCol.Text);
    TableW := 5500;
    case Dlg.RadioGroup.ItemIndex of
      0: TableW := 4000;
      1: TableW := 5500;
      2: TableW := 7000;
    end;
  finally
    Dlg.Release;
  end;

  W := TableW div Cols;

  SL := TStringList.Create;
  try
    SL.Add(Head);

    for I := 0 to Rows -1 do
    begin
      SL.Add(Row);

      for J := 0 to Cols -1 do
      begin
        SL[SL.Count -1] := SL[SL.Count -1] + Col;
        SL[SL.Count -1] := SL[SL.Count -1] + Format('%d', [W * (J + 1)]);
      end;
      SL[SL.Count -1] := SL[SL.Count -1] + '\pard\intbl';
      for J := 0 to Cols -1 do
      begin
        SL[SL.Count -1] := SL[SL.Count -1] + Endcell;
      end;
      SL[SL.Count -1] := SL[SL.Count -1] + Endrow;
    end;
    //SL[SL.Count -1] := SL[SL.Count -1] + '\par}';

    Rich := TJvRichEdit.Create(nil);
    try
      Rich.Visible := False;
      Rich.Parent := Self;
      Rich.StreamFormat := sfRichText;
      Rich.Lines.Add(SL.Text);

      MS := TMemoryStream.Create;
      try
        MS.Position := 0;
        Rich.Lines.SaveToStream(MS);
        MS.Position := 0;

        ES.dwCookie := DWORD(MS);
        ES.dwError := 0;
        ES.pfnCallback := @myCallback;

        Editor.Perform(EM_STREAMIN, SF_RTF or SFF_SELECTION, LParam(@ES));
      finally
        MS.Free;
      end;
    finally
      Rich.Free;
    end;
  finally
    SL.Free;
  end;
end;


//----------------------------------------
// tHg_CAO
//----------------------------------------
procedure TEditorForm.ActionFontDlgExecute(Sender: TObject);
begin
  FontDialog.Font.Assign(Editor.SelAttributes);
  if FontDialog.Execute then
  begin
    Editor.SelAttributes.Assign(FontDialog.Font);
  end;
end;


//----------------------------------------
// Rs[
//----------------------------------------
procedure TEditorForm.ActionEditCopyExecute(Sender: TObject);
begin
  Editor.CopyToClipboard;
end;

procedure TEditorForm.ActionEditCopyUpdate(Sender: TObject);
begin
  ActionEditCopy.Enabled := Editor.SelLength > 0;
end;


//----------------------------------------
// ؂
//----------------------------------------
procedure TEditorForm.ActionEditCutExecute(Sender: TObject);
begin
  Editor.CutToClipboard;
end;

procedure TEditorForm.ActionEditCutUpdate(Sender: TObject);
begin
  ActionEditCut.Enabled := Editor.SelLength > 0;
end;


//----------------------------------------
// \t
//----------------------------------------
procedure TEditorForm.ActionEditPasteExecute(Sender: TObject);
begin
  Editor.PasteFromClipboard;
end;

procedure TEditorForm.ActionEditPasteUpdate(Sender: TObject);
begin
  ActionEditPaste.Enabled := Editor.CanPaste;
end;


//----------------------------------------
// `Iē\t
//----------------------------------------
procedure TEditorForm.ActionEditFormatPasteExecute(Sender: TObject);
begin
  Editor.PasteSpecialDialog;
end;

procedure TEditorForm.ActionEditFormatPasteUpdate(Sender: TObject);
begin
  ActionEditFormatPaste.Enabled := Editor.CanPaste;
end;


//----------------------------------------
// ɖ߂
//----------------------------------------
procedure TEditorForm.ActionEditUndoExecute(Sender: TObject);
begin
  Editor.Undo;
end;

procedure TEditorForm.ActionEditUndoUpdate(Sender: TObject);
begin
  ActionEditUndo.Enabled := Editor.CanUndo;
end;


//----------------------------------------
// 蒼
//----------------------------------------
procedure TEditorForm.ActionEditRedoExecute(Sender: TObject);
begin
  Editor.Redo;
end;

procedure TEditorForm.ActionEditRedoUpdate(Sender: TObject);
begin
  ActionEditRedo.Enabled := Editor.CanRedo;
end;


//----------------------------------------
// SđI
//----------------------------------------
procedure TEditorForm.ActionEditSelectAllExecute(Sender: TObject);
begin
  Editor.SelectAll;
end;


//----------------------------------------
// }EXړ
//----------------------------------------
procedure TEditorForm.EditorMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  S: String;
begin
  // m[hNǂׂ
  if IsNodeLink(Point(X, Y), S) then
  begin
    Editor.Cursor := crHandPoint;
  end
  else
  begin
    Editor.Cursor := crDefault;
  end;
end;


//----------------------------------------
// GfB^_uNbN
//----------------------------------------
procedure TEditorForm.EditorDblClick(Sender: TObject);
var
  Pt: TPoint;
  S: String;
begin
  GetCursorPos(Pt);
  Pt := Editor.ScreenToClient(Pt);

  // Nǂׂ
  if IsNodeLink(Pt, S) then
  begin
    // m[hɃNĂÃm[hɈړ
    MainForm.JumpToNode(S);
  end;
end;


end.
