unit FormPrintPreview;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Printers, Math, Dialogs, JvExControls, JvComponent, JvPrvwDoc, JvPrvwRender,
  ExtCtrls, ActnList, ComCtrls, StdCtrls, JvExStdCtrls, JvRichEdit,
  ImgList, JvButton, JvCtrls, Mask, JvExMask, JvSpin, AppEvnts,
  JvAppStorage, JvAppIniStorage, JvComponentBase;

type
  TPrintPreviewForm = class(TForm)
    PreviewControl: TJvPreviewControl;
    PreviewRenderRichEdit: TJvPreviewRenderJvRichEdit;
    PrinterSetupDialog: TPrinterSetupDialog;
    PrintDialog: TPrintDialog;
    ActionList1: TActionList;
    StatusBar: TStatusBar;
    ActionPrintSetting: TAction;
    ActionPrint: TAction;
    PreviewPrinter: TJvPreviewPrinter;
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    PreviewEdit: TJvRichEdit;
    TopPanel: TPanel;
    ButtonPrint: TJvImgBtn;
    ButtonSetting: TJvImgBtn;
    ImageList: TImageList;
    ScaleComboBox: TComboBox;
    Label1: TLabel;
    ZoomSpinEdit: TJvSpinEdit;
    Label2: TLabel;
    ColsSpinEdit: TJvSpinEdit;
    RowsSpinEdit: TJvSpinEdit;
    Label3: TLabel;
    Label4: TLabel;
    ApplicationEvents: TApplicationEvents;
    ButtonClose: TJvImgBtn;
    IniStorage: TJvAppIniFileStorage;
    procedure FormCreate(Sender: TObject);
    procedure ActionPrintSettingExecute(Sender: TObject);
    procedure ActionPrintExecute(Sender: TObject);
    procedure PreviewControlAfterScroll(Sender: TObject);
    procedure PreviewControlChange(Sender: TObject);
    procedure ScaleComboBoxChange(Sender: TObject);
    procedure ZoomSpinEditChange(Sender: TObject);
    procedure ColsSpinEditChange(Sender: TObject);
    procedure RowsSpinEditChange(Sender: TObject);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure ButtonCloseClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private 錾 }
  public
    { Public 錾 }
    procedure SetPreview;
  end;

var
  PrintPreviewForm: TPrintPreviewForm;

implementation

{$R *.dfm}


//----------------------------------------
// vr[ύXۂ̍Đݒ
//----------------------------------------
procedure TPrintPreviewForm.SetPreview;
begin
  PreviewControl.DeviceInfo.ReferenceHandle := Printer.Handle;

  // }[W0.5inchɐݒ
  PreviewControl.DeviceInfo.OffsetLeft :=Max(
    PreviewControl.DeviceInfo.InchToXPx(0.5),
    PreviewControl.DeviceInfo.OffsetLeft);
  PreviewControl.DeviceInfo.OffsetRight := Max(
    PreviewControl.DeviceInfo.InchToXPx(0.5),
    PreviewControl.DeviceInfo.OffsetRight);
  PreviewControl.DeviceInfo.OffsetTop := Max(
    PreviewControl.DeviceInfo.InchToYPx(0.5),
    PreviewControl.DeviceInfo.OffsetTop);
  PreviewControl.DeviceInfo.OffsetBottom := Max(
    PreviewControl.DeviceInfo.InchToYPx(0.5),
    PreviewControl.DeviceInfo.OffsetBottom);

  // vr[č쐬
  PreviewRenderRichEdit.CreatePreview(False);
end;


//----------------------------------------
// tH[̏
//----------------------------------------
procedure TPrintPreviewForm.FormCreate(Sender: TObject);
begin
  // vr[tH[̏ǂݍ
  IniStorage.Reload;
  IniStorage.Path := 'PreviewForm';
  Self.Width := IniStorage.ReadInteger('Width', Trunc(Screen.Width * 0.8));
  Self.Height := IniStorage.ReadInteger('Height', Trunc(Screen.Height * 0.8));
  if IniStorage.ReadBoolean('Maximized', False) then
  begin
    Self.WindowState := wsMaximized;
  end
  else
  begin
    Self.WindowState := wsNormal;
  end;

  SetPreview;
  PreviewControl.Options.ScaleMode := TJvPreviewScaleMode(ScaleComboBox.ItemIndex);
  PreviewControl.OnChange := PreviewControlChange;
end;


//----------------------------------------
// tH[j
//----------------------------------------
procedure TPrintPreviewForm.FormDestroy(Sender: TObject);
begin
  // tH[̏L^
  IniStorage.Path := 'PreviewForm';
  if Self.WindowState = wsMaximized then
  begin
    IniStorage.WriteBoolean('Maximized', True);
  end
  else
  begin
    IniStorage.WriteBoolean('Maximized', False);
    IniStorage.WriteInteger('Width', Self.Width);
    IniStorage.WriteInteger('Height', Self.Height);
  end;
  IniStorage.Flush;
end;


//----------------------------------------
// ݒ
//----------------------------------------
procedure TPrintPreviewForm.ActionPrintSettingExecute(Sender: TObject);
begin
  PrinterSetupDialog.Execute;
  SetPreview;
end;


//----------------------------------------
// 
//----------------------------------------
procedure TPrintPreviewForm.ActionPrintExecute(Sender: TObject);
begin
  PrintDialog.PrintRange := prAllPages;

  if PreviewControl.PageCount < 1 then
  begin
    PrintDialog.Options := PrintDialog.Options - [poPageNums]
  end
  else
  begin
    PrintDialog.Options := PrintDialog.Options + [poPageNums];
    PrintDialog.FromPage := 1;
    PrintDialog.ToPage := PreviewControl.PageCount;
    PrintDialog.MinPage := 1;
    PrintDialog.MaxPage := PreviewControl.PageCount;
  end;

  if PrintDialog.Execute then
  begin
    PreviewPrinter.Assign(PrintDialog);
    PreviewPrinter.Printer := Printer;
    PreviewPrinter.Print;
  end;
end;


//----------------------------------------
// vr[XN[
//----------------------------------------
procedure TPrintPreviewForm.PreviewControlAfterScroll(Sender: TObject);
begin
  Statusbar.Panels[1].Text :=
    Format(': %d,@i: %d,@%d i',
    [PreviewControl.TotalCols, PreviewControl.VisibleRows, PreviewControl.TopRow +1]);
end;


//----------------------------------------
// vr[ύXꂽ
//----------------------------------------
procedure TPrintPreviewForm.PreviewControlChange(Sender: TObject);
begin
  ScaleComboBox.ItemIndex := Ord(PreviewControl.Options.ScaleMode);
  ZoomSpinEdit.Value := PreviewControl.Options.Scale;
  ColsSpinEdit.Value := PreviewControl.Options.Cols;
  RowsSpinEdit.Value := PreviewControl.Options.Rows;

  Statusbar.Panels[0].Text := Format('%d y[W', [PreviewControl.PageCount]);
  Statusbar.Panels[1].Text :=
    Format(': %d,@i: %d,@%d i',
    [PreviewControl.TotalCols, PreviewControl.VisibleRows, PreviewControl.TopRow +1]);
end;


procedure TPrintPreviewForm.ScaleComboBoxChange(Sender: TObject);
begin
  PreviewControl.Options.ScaleMode := TJvPreviewScaleMode(ScaleComboBox.ItemIndex);
  ScaleComboBox.ItemIndex := Ord(PreviewControl.Options.ScaleMode);
end;


procedure TPrintPreviewForm.ZoomSpinEditChange(Sender: TObject);
begin
  PreviewControl.Options.Scale := StrToInt(ZoomSpinEdit.Text);
  ZoomSpinEdit.Value := PreviewControl.Options.Scale;
end;


procedure TPrintPreviewForm.ColsSpinEditChange(Sender: TObject);
begin
  PreviewControl.Options.Cols := StrToInt(ColsSpinEdit.Text);
  ColsSpinEdit.Value := PreviewControl.Options.Cols;
end;


procedure TPrintPreviewForm.RowsSpinEditChange(Sender: TObject);
begin
  PreviewControl.Options.Rows := StrToInt(RowsSpinEdit.Text);
  RowsSpinEdit.Value := PreviewControl.Options.Rows;
end;


procedure TPrintPreviewForm.ApplicationEventsIdle(Sender: TObject;
  var Done: Boolean);
begin
  ZoomSpinEdit.Enabled :=
    (PreviewControl.Options.ScaleMode = smScale) or
    (PreviewControl.Options.ScaleMode = smAutoScale);
  ColsSpinEdit.Enabled :=
    (PreviewControl.Options.ScaleMode = smScale) or
    (PreviewControl.Options.ScaleMode = smColsRows);
  RowsSpinEdit.Enabled :=
    (PreviewControl.Options.ScaleMode = smScale) or
    (PreviewControl.Options.ScaleMode = smColsRows);
end;


procedure TPrintPreviewForm.ButtonCloseClick(Sender: TObject);
begin
  Close;
end;


end.
