unit FormTree;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  ActiveX, ShellAPI, Dialogs, JvDockTree, JvDockControlForm, JvDockVIDStyle, JvDockVSNetStyle,
  JvComponentBase, VirtualTrees, ImgList, ActnList, Zlib;

type
  // m[hf[^
  PNodeData = ^TNodeData;
  TNodeData = record
    IsCreated: Boolean;
    Caption: string;
    SelStart: Integer;
    SelLength: Integer;
    StateIndex : Integer;
    Strings : TStrings;
    NodeNo: Integer;
  end;

  TTreeForm = class(TForm)
    JvDockClient: TJvDockClient;
    JvDockVSNetStyle: TJvDockVSNetStyle;
    TreeView: TVirtualStringTree;
    TreeImageList: TImageList;
    TreeActionList: TActionList;
    ActionInsertNode: TAction;
    ActionAddNode: TAction;
    ActionAddChildNode: TAction;
    ActionDeteleNode: TAction;
    ActionMovePrev: TAction;
    ActionMoveNext: TAction;
    ActionMoveDown: TAction;
    ActionMoveUp: TAction;
    ActionCopy: TAction;
    ActionCut: TAction;
    ActionPaste: TAction;
    ActionExpandAll: TAction;
    ActionPrevFocus: TAction;
    ActionNextFocus: TAction;
    ActionSortNode: TAction;
    ActionChangeCaption: TAction;
    ImageList: TImageList;
    StateImageList: TImageList;
    procedure TreeViewDragOver(Sender: TBaseVirtualTree; Source: TObject;
      Shift: TShiftState; State: TDragState; Pt: TPoint; Mode: TDropMode;
      var Effect: Integer; var Accept: Boolean);
    procedure TreeViewDragDrop(Sender: TBaseVirtualTree; Source: TObject;
      DataObject: IDataObject; Formats: TFormatArray; Shift: TShiftState;
      Pt: TPoint; var Effect: Integer; Mode: TDropMode);
    procedure TreeViewGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure TreeViewCollapsed(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure ActionInsertNodeExecute(Sender: TObject);
    procedure ActionAddNodeExecute(Sender: TObject);
    procedure ActionAddChildNodeExecute(Sender: TObject);
    procedure ActionDeteleNodeExecute(Sender: TObject);
    procedure ActionDeteleNodeUpdate(Sender: TObject);
    procedure ActionMovePrevExecute(Sender: TObject);
    procedure ActionMovePrevUpdate(Sender: TObject);
    procedure ActionMoveNextExecute(Sender: TObject);
    procedure ActionMoveNextUpdate(Sender: TObject);
    procedure ActionMoveDownExecute(Sender: TObject);
    procedure ActionMoveDownUpdate(Sender: TObject);
    procedure ActionMoveUpExecute(Sender: TObject);
    procedure ActionMoveUpUpdate(Sender: TObject);
    procedure ActionCopyExecute(Sender: TObject);
    procedure ActionCopyUpdate(Sender: TObject);
    procedure ActionCutExecute(Sender: TObject);
    procedure ActionCutUpdate(Sender: TObject);
    procedure ActionPasteExecute(Sender: TObject);
    procedure ActionPasteUpdate(Sender: TObject);
    procedure ActionExpandAllExecute(Sender: TObject);
    procedure ActionExpandAllUpdate(Sender: TObject);
    procedure ActionMergeNodeUpdate(Sender: TObject);
    procedure ActionChangeCaptionExecute(Sender: TObject);
    procedure TreeViewCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure TreeViewGetNodeDataSize(Sender: TBaseVirtualTree;
      var NodeDataSize: Integer);
    procedure TreeViewGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure TreeViewInitNode(Sender: TBaseVirtualTree; ParentNode,
      Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
    procedure TreeViewFreeNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure TreeViewNewText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; NewText: WideString);
    procedure TreeViewEdited(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex);
    procedure TreeViewFocusChanging(Sender: TBaseVirtualTree; OldNode,
      NewNode: PVirtualNode; OldColumn, NewColumn: TColumnIndex;
      var Allowed: Boolean);
    procedure TreeViewFocusChanged(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex);
    procedure TreeViewLoadNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Stream: TStream);
    procedure TreeViewSaveNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Stream: TStream);
    procedure FormCreate(Sender: TObject);
    procedure ActionSortNodeExecute(Sender: TObject);
    procedure ActionSortNodeUpdate(Sender: TObject);
    procedure ActionPrevFocusExecute(Sender: TObject);
    procedure ActionNextFocusExecute(Sender: TObject);
    procedure TreeViewPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType);
  private
    { Private 錾 }
    // m[h
    procedure InitNode(Sender: TBaseVirtualTree; Node: PVirtualNode);

    // Nanat@C̑}
    procedure AddTreeFromNanaFile(const Path: TFileName; TargetNode: PVirtualNode;
      Mode: TVTNodeAttachMode);
    // KweLXgt@CAeLXgt@CAb`eLXgt@C̑}
    procedure AddTreeFromTreeTextFile(const Path: TFileName; TargetNode: PVirtualNode;
      Mode: TVTNodeAttachMode);
    // OLEf[^}
    procedure AddTreeFromOleFile(DataObject: IDataObject; Target: TVirtualStringTree;
      Mode: TVTNodeAttachMode);
    // f[^}
    procedure InsertData(Sender: TBaseVirtualTree; DataObject: IDataObject; Formats: TFormatArray; Effect: Integer;
      Mode: TVTNodeAttachMode);
    // \[g
    procedure SortNode(Node: PVirtualNode; Recursive: Boolean; SortDirection: TSortDirection);
  public
    { Public 錾 }
    NodeNo: Integer;
    Modified: Boolean;
    // ANZT֐
    function NodeData(Node: PVirtualNode): PNodeData;
    // t@Cf[^}
    procedure AddTreeFromFile(Path: TFileName; Node: PVirtualNode; Mode: TVTNodeAttachMode);
    // Nana`ŕۑ
    procedure SaveToNanaFile(const Path: TFileName);
  end;

var
  TreeForm: TTreeForm;

implementation

uses StrUtils, ModuleGlobal, FormMain, DlgSort;

{$R *.dfm}


//----------------------------------------
// k
//----------------------------------------
procedure CompressStream(inpStream, outStream: TStream);
var
  InpBuf, OutBuf: Pointer;
  InpBytes, OutBytes: Integer;
begin
  InpBuf := nil;
  OutBuf := nil;
  try
    GetMem(InpBuf, inpStream.Size);
    inpStream.Position := 0;
    InpBytes := inpStream.Read(InpBuf^, inpStream.Size);
    CompressBuf(InpBuf, InpBytes, OutBuf, OutBytes);
    outStream.Write(OutBuf^, OutBytes);
  finally
    if InpBuf <> nil then FreeMem(InpBuf);
    if OutBuf <> nil then FreeMem(OutBuf);
  end;
end;


//----------------------------------------
// WJ
//----------------------------------------
procedure DecompressStream(inpStream, outStream: TStream);
var
  InpBuf, OutBuf: Pointer;
  OutBytes, sz: Integer;
begin
  InpBuf := nil;
  OutBuf := nil;
  sz     := inpStream.Size - inpStream.Position;
  if sz > 0 then
    try
      GetMem(InpBuf, sz);
      inpStream.Read(InpBuf^, sz);
      DecompressBuf(InpBuf, sz, 0, OutBuf, OutBytes);
      outStream.Write(OutBuf^, OutBytes);
    finally
      if InpBuf <> nil then FreeMem(InpBuf);
      if OutBuf <> nil then FreeMem(OutBuf);
    end;
  outStream.Position := 0;
end;


//----------------------------------------
// ANZT֐
//----------------------------------------
function TTreeForm.NodeData(Node: PVirtualNode): PNodeData;
begin
  Result := TreeView.GetNodeData(Node);
end;


//----------------------------------------
// m[h
//----------------------------------------
procedure TTreeForm.InitNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
const
  sCaption = 'VKm[h';
var
  Data: PNodeData;
begin
  Data := NodeData(Node);

  if not Data.IsCreated then
  begin
    Data.Caption := sCaption;
    Data.StateIndex := -1;
    Data.SelStart := 0;
    Data.SelLength := 0;
    Data.Strings := TStringList.Create;
    Data.NodeNo := NodeNo;
    Inc(NodeNo);
    Data.IsCreated := True;
    Modified := True;
  end;
end;


//----------------------------------------
// c[Nana`̃t@Cǉ
//----------------------------------------
const
  // Nana`t@C̃^O
  sChapter   = '<chapter>';
  sAttr      = '<attribute>';
  sGroup     = '<group>';
  sUngroup   = '</group>';
  sContent   = '<content>';
  sUncontent = '</content>';

procedure TTreeForm.AddTreeFromNanaFile(const Path: TFileName; TargetNode: PVirtualNode;
  Mode: TVTNodeAttachMode);
var
  SL: TStrings;
  I: Integer;
  Node: PVirtualNode;
  FocusedNode: PVirtualNode;
  MSIn: TMemoryStream;
  MSOut: TMemoryStream;

  // 葱(c[ڂ̑Zbg)
  procedure SetNodeAttributes;
  var
    Attr: String;
    AttrSL: TStringList;
  begin
    Attr := Copy(SL[I], Length(sAttr) + 1, Length(SL[I]) - Length(sAttr));
    AttrSL := TStringList.Create;
    try
      AttrSL.CommaText := Attr;

      // 1ڂ̑(C[Wԍ)ǂݍ
      if AttrSL.Count > 0 then
      begin
        NodeData(Node).StateIndex := StrToInt(AttrSL[0]);
      end;

      GlobalModule.IniStorage.Path := 'General';
      if True = GlobalModule.IniStorage.ReadBoolean('RestoreTree', False) then
      begin
        // 2ڂ̑(m[h̊J)ǂݍ
        if AttrSL.Count > 1 then
        begin
          if StrToInt(AttrSL[1]) = 1 then
          begin
            if not TreeView.Expanded[Node.Parent] then
            begin
              TreeView.Expanded[Node.Parent] := True;
            end;
          end;
        end;
        // 3ڂ̑(tH[JX̗L)ǂݍ
        if AttrSL.Count > 2 then
        begin
          if StrToInt(AttrSL[2]) = 1 then
          begin
            FocusedNode := Node;
          end;
        end;
      end;
    finally
      AttrSL.Free;
    end;
  end;

begin
  FocusedNode := nil;
  SL := TStringList.Create;
  Screen.Cursor := crHourGlass;
  try
  try
    // Ver.105 kNanat@C(*.nnz)Ή 
    if ExtractFileExt(Path) = '.nnz' then
    begin
      MSIn := TMemoryStream.Create;
      MSOut := TMemoryStream.Create;
      try
        MSIn.LoadFromFile(Path);
        DecompressStream(MSIn, MSOut);
        SL.LoadFromStream(MSOut);
      finally
        MSOut.Free;
        MSIn.Free;
      end;
    end
    else
    begin
      SL.LoadFromFile(Path);
    end;
    // Ver.105 kNanat@C(*.nnz)Ή ܂

    I := 0;
    Node := TargetNode;

    while I < SL.Count do
    begin
      if (SL[I] = sGroup) then
      begin
        Inc(I);
        if AnsiStartsStr(sChapter, SL[I]) then
        begin

          if Node = TargetNode then
          begin
            Node := TreeView.InsertNode(Node, Mode);
          end
          else
          begin
            Node := TreeView.AddChild(Node);
          end;
          InitNode(TreeView, Node);
          TreeView.Text[Node, 0] :=
            Copy(SL[I], Length(sChapter) + 1, Length(SL[I]) - Length(sChapter));

          Inc(I);
          while not AnsiStartsText(sAttr, SL[I]) do
          begin
            Inc(I);
          end;
          SetNodeAttributes;

        end;
      end
      else if AnsiStartsStr(sChapter, SL[I]) then
      begin

        if Node = TargetNode then
        begin
          Node := TreeView.InsertNode(Node, Mode);
        end
        else
        begin
          Node := TreeView.InsertNode(Node, amInsertAfter);
        end;
        InitNode(TreeView, Node);
        TreeView.Text[Node, 0] :=
          Copy(SL[I], Length(sChapter) + 1, Length(SL[I]) - Length(sChapter));

        Inc(I);
        while not AnsiStartsText(sAttr, SL[I]) do
        begin
          Inc(I);
        end;
        SetNodeAttributes;

      end
      else if (SL[I] = sContent) then
      begin
      end
      else if (SL[I] = sUncontent) then
      begin
      end
      else if (SL[I] = sUnGroup) then
      begin
        Node := Node.Parent;
      end
      else
      begin
        if Node = TargetNode then
        begin
          raise EReadError.Create('ȃt@Cǂݍ񂾂݂');
        end;
        NodeData(Node).Strings.Add(SL[I]);
      end;
      Inc(I);
    end;

    GlobalModule.IniStorage.Path := 'General';
    if (True = GlobalModule.IniStorage.ReadBoolean('RestoreTree', False)) and
      Assigned(FocusedNode) then
    begin
      TreeView.FocusedNode := FocusedNode;
    end
    else
    begin
      TreeView.FocusedNode := TreeView.RootNode.FirstChild;
    end;
    TreeView.Selected[TreeView.FocusedNode] := True;

  finally
    Screen.Cursor := crDefault;
    SL.Free;
  end;
  except
    if mrYes = MessageDlg(Path + 'nanat@CƂēǂݍ߂܂łB' + #13#10 +
                          'eLXgt@CƂēǂݍł݂܂H',
                          mtWarning, [mbYes, mbNo], 0) then
    begin
      AddTreeFromTreeTextFile(Path, TargetNode, Mode);
    end;
  end;
end;


//----------------------------------------
// c[ɊKweLXg`̃t@Cǉ
// (eLXg`Ab`eLXg`OK)
//----------------------------------------
procedure TTreeForm.AddTreeFromTreeTextFile(const Path: TFileName; TargetNode: PVirtualNode;
  Mode: TVTNodeAttachMode);
var
  SL: TStrings;
  I: Integer;
  J: Integer;
  Tier: Integer;
  DotCount: Integer;
  Node: PVirtualNode;
begin
  SL := TStringList.Create;
  Screen.Cursor := crHourGlass;
  try
    SL.LoadFromFile(Path);

    Tier := 1;
    I := 0;
    Node := TargetNode;

    while I < SL.Count do
    begin
      // m[hs̏ꍇ
      if AnsiStartsStr('.', SL[I]) then
      begin
        // sIh̐𒲂ׂ
        DotCount := 0;
        while True do
        begin
          if Copy(SL[I], DotCount, 1) = '.' then
          begin
            DotCount := DotCount + 1;
          end
          else
          begin
            Break;
          end;
        end;

        // ݂̊KwƓKwȂAKwɃm[hǉ
        if Tier = DotCount then
        begin
          if Node = TargetNode then
          begin
            Node := TreeView.InsertNode(Node, Mode);
          end
          else
          begin
            Node := TreeView.InsertNode(Node, amInsertAfter);
          end;
          InitNode(TreeView, Node);
        end

        // ݂̊Kw[KwȂÅKwɃm[hǉ
        else if Tier < DotCount then
        begin
          for J := 1 to (DotCount - Tier) do
          begin
            if Node = TargetNode then
            begin
              Node := TreeView.InsertNode(Node, Mode);
            end
            else
            begin
              Node := TreeView.AddChild(Node);
            end;
            InitNode(TreeView, Node);
          end;
        end

        // ݂̊Kw󂢊KwȂAɏオăm[hǉ
        else
        begin
          for J := 1 to (Tier - DotCount) do
          begin
            Node := Node.Parent;
          end;
          if Node = TargetNode then
          begin
            Node := TreeView.InsertNode(Node, Mode);
          end
          else
          begin
            Node := TreeView.InsertNode(Node, amInsertAfter);
          end;
          InitNode(TreeView, Node);
        end;

        TreeView.Text[Node, 0] := Copy(SL[I], DotCount, Length(SL[I]) - DotCount + 1);
        Tier := DotCount;
      end

      // {̏ꍇ
      else
      begin
        if Node = TargetNode then
        begin
          Node := TreeView.InsertNode(Node, Mode);
          InitNode(TreeView, Node);
        end;

        if not Assigned(Node) then
        begin
          Node := TreeView.AddChild(Node);
          InitNode(TreeView, Node);
        end;

        NodeData(Node).Strings.Add(SL[I]);
      end;
      Inc(I);
    end;
  finally
    Screen.Cursor := crDefault;
    SL.Free;
  end;
end;


//----------------------------------------
// t@Cf[^}
//----------------------------------------
procedure TTreeForm.AddTreeFromFile(Path: TFileName; Node: PVirtualNode;
  Mode: TVTNodeAttachMode);
begin
  if not FileExists(Path) then
  begin
    MessageDlg('t@C܂ł', mtWarning, [mbOK], 0);
    Exit;
  end;

  // gqNanat@C(*.nna)AkNanat@C(*.nnz)
  // ȂANanat@Cǂݍ
  if (ExtractFileExt(Path) = '.nna') or
     (ExtractFileExt(Path) = '.nnz') then
  begin
    AddTreeFromNanaFile(Path, Node, Mode);
  end
  // gqNanat@CȊOȂAƂ肠ǂݍł݂
  else
  begin
    AddTreeFromTreeTextFile(Path, Node, Mode);
  end;
end;


//----------------------------------------
// OLEf[^}
//----------------------------------------
procedure TTreeForm.AddTreeFromOleFile(DataObject: IDataObject; Target: TVirtualStringTree;
  Mode: TVTNodeAttachMode);
var
  Medium: TStgMedium;
  FormatEtc: TFormatEtc;
  I, Count: Integer;
  szFile: array[0..255] of Char;
  TargetNode: PVirtualNode;
begin
  FormatEtc.cfFormat := CF_HDROP;
  FormatEtc.ptd := nil;
  FormatEtc.dwAspect := DVASPECT_CONTENT;
  FormatEtc.lindex := -1;
  FormatEtc.tymed := TYMED_HGLOBAL;

  if DataObject.QueryGetData(FormatEtc) = S_OK then
  begin
    if DataObject.GetData(FormatEtc, Medium) = S_OK then
    try
      Count := DragQueryFile(Medium.hGlobal, $FFFFFFFF, Nil, 0);
      if Count > 0 then
      begin
        Target.BeginUpdate;
        try
          TargetNode := Target.DropTargetNode;
          if TargetNode = nil then
          begin
            TargetNode := Target.FocusedNode;
          end;
          for I := Count -1 downto 0 do
          begin
            // t@CpXoāAǂݍ
            DragQueryFile(Medium.hGlobal, I, szFile, SizeOf(szFile));
            AddTreeFromFile(szFile, TargetNode, Mode);
          end;
        finally
          Target.EndUpdate;
        end;
      end;
    finally
      ReleaseStgMedium(Medium);
    end;
  end;
end;


//----------------------------------------
// f[^}
//----------------------------------------
procedure TTreeForm.InsertData(Sender: TBaseVirtualTree; DataObject: IDataObject; Formats: TFormatArray;
  Effect: Integer; Mode: TVTNodeAttachMode);
var
  I: Integer;
begin
  for I := 0 to High(Formats) do
  begin
    // c[r[lCeBuf[^`̏ꍇ
    if Formats[I] = CF_VIRTUALTREE then
    begin
      if Sender.ProcessDrop(DataObject, Sender.DropTargetNode, Effect, Mode) then
      begin
        Modified := True;
      end;
      Exit;
    end
    // t@CXg̏ꍇ
    else if Formats[I] = CF_HDROP then
    begin
      AddTreeFromOleFile(DataObject, Sender as TVirtualStringTree, Mode);
      Exit;
    end;
  end;
end;


//----------------------------------------
// \[g
//----------------------------------------
procedure TTreeForm.SortNode(Node: PVirtualNode; Recursive: Boolean;
  SortDirection: TSortDirection);
var
  ChildNode: PVirtualNode;
begin
  // \[g
  TreeView.Sort(Node, -1, SortDirection);

  // ċA\[g̏ꍇAɃ\[g
  if Recursive then
  begin
    if Node.ChildCount > 0 then
    begin
      ChildNode := Node.FirstChild;
      while Assigned(ChildNode) do
      begin
        SortNode(ChildNode, Recursive, SortDirection);
        ChildNode := ChildNode.NextSibling;
      end;
    end;
  end;
end;


//----------------------------------------
// m[hhbO
//----------------------------------------
procedure TTreeForm.TreeViewDragOver(Sender: TBaseVirtualTree;
  Source: TObject; Shift: TShiftState; State: TDragState; Pt: TPoint;
  Mode: TDropMode; var Effect: Integer; var Accept: Boolean);
begin
  Accept := True;
end;


//----------------------------------------
// m[hhbv
//----------------------------------------
procedure TTreeForm.TreeViewDragDrop(Sender: TBaseVirtualTree;
  Source: TObject; DataObject: IDataObject; Formats: TFormatArray;
  Shift: TShiftState; Pt: TPoint; var Effect: Integer; Mode: TDropMode);
var
  Attachmode: TVTNodeAttachMode;
  Nodes: TNodeArray;
begin
  Nodes := nil;

  // ǂ̈ʒuɃhbv邩߂
  case Mode of
    dmAbove:  AttachMode := amInsertBefore;
    dmOnNode: AttachMode := amAddChildLast;
    dmBelow:  AttachMode := amInsertAfter;
  else
    AttachMode := amNowhere;
  end;

  // Rs[ړ߂
  if Source is TBaseVirtualTree then
  begin
    if ssCtrl in Shift then
    begin
      Effect := DROPEFFECT_COPY
    end
    else
    begin
      Effect := DROPEFFECT_MOVE;
    end;
  end;

  InsertData(Sender, DataObject, Formats, Effect, AttachMode);
end;


//----------------------------------------
// m[h̃C[WԍKvɂȂƂɌĂяo
//----------------------------------------
procedure TTreeForm.TreeViewGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer);
begin
  // Xe[gC[W
  if (Kind = ikState) then
  begin
    ImageIndex := NodeData(Node).StateIndex;
  end
  // m[hC[W
  else
  begin
    // m[hɎqꍇ́A{̃C[W
    if Node.ChildCount > 0 then
    begin
      if Sender.Expanded[Node] then
      begin
        ImageIndex := 2;
      end
      else
      begin
        ImageIndex := 1;
      end;
    end
    // ̃C[W
    else
    begin
      ImageIndex := 0;
    end;
  end;
end;


//----------------------------------------
// c[܂肽񂾂Ƃ
//----------------------------------------
procedure TTreeForm.TreeViewCollapsed(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
begin
  TreeView.ClearSelection;
  TreeView.FocusedNode := Node;
  TreeView.Selected[Node] := True;
end;


//----------------------------------------
// m[h}
//----------------------------------------
procedure TTreeForm.ActionInsertNodeExecute(Sender: TObject);
var
  Node: PVirtualNode;
begin
  Node := TreeView.InsertNode(TreeView.FocusedNode, amInsertBefore);
  TreeView.ClearSelection;
  TreeView.FocusedNode := Node;
  TreeView.Selected[Node] := True;

  Modified := True;

  MainForm.TreeViewInsertNode(Sender);
end;


//----------------------------------------
// m[hǉ
//----------------------------------------
procedure TTreeForm.ActionAddNodeExecute(Sender: TObject);
var
  Node: PVirtualNode;
begin
  Node := TreeView.InsertNode(TreeView.FocusedNode, amInsertAfter);
  TreeView.ClearSelection;
  TreeView.FocusedNode := Node;
  TreeView.Selected[Node] := True;

  Modified := True;

  MainForm.TreeViewInsertNode(Sender);
end;


//----------------------------------------
// qm[hǉ
//----------------------------------------
procedure TTreeForm.ActionAddChildNodeExecute(Sender: TObject);
var
  Node: PVirtualNode;
begin
  Node := TreeView.InsertNode(TreeView.FocusedNode, amAddChildLast);
  TreeView.ClearSelection;
  TreeView.FocusedNode := Node;
  TreeView.Selected[Node] := True;

  Modified := True;

  MainForm.TreeViewInsertNode(Sender);
end;


//----------------------------------------
// m[h폜
//----------------------------------------
procedure TTreeForm.ActionDeteleNodeExecute(Sender: TObject);
var
  Nodes: TNodeArray;
  Node: PVirtualNode;
begin
  Nodes := nil;
  Node := nil;

  GlobalModule.IniStorage.Path := 'Tree';
  if True = GlobalModule.IniStorage.ReadBoolean('Warning', True) then
  begin
    if mrYes <> MessageDlg(
      'm[h폜܂B̑͌ɖ߂܂B'#13#10'낵łH',
      mtConfirmation, [mbYes, mbNo], 0) then Exit;
  end;

  // tH[JX̂m[hIm[hɊ܂܂ĂƁA
  // 폜ɃtH[JXȂĂ܂̂ŁA
  // 炽ɃtH[JXm[hݒ肷
  if TreeView.Selected[TreeView.FocusedNode] then
  begin
    Nodes := TreeView.GetSortedSelection(False);

    // ܂́AԍŌɑIĂm[h݂̎
    Node := Nodes[High(Nodes)].NextSibling;
    // ɁAԍŏɑIĂm[h̑O݂
    if Node = nil then
    begin
      Node := Nodes[Low(Nodes)].PrevSibling;
    end;
    // OAem[h݂
    if Node = nil then
    begin
      Node := Nodes[Low(Nodes)].Parent;
    end;
  end;

  TreeView.DeleteSelectedNodes;

  if TreeView.RootNodeCount = 0 then
  begin
    Node := TreeView.InsertNode(nil, amInsertAfter);
  end;

  if nil = TreeView.FocusedNode then
  begin
    TreeView.FocusedNode := Node;
  end;

  Modified := True;
end;

procedure TTreeForm.ActionDeteleNodeUpdate(Sender: TObject);
begin
  ActionDeteleNode.Enabled := TreeView.SelectedCount > 0;
end;


//----------------------------------------
// Oֈړ
//----------------------------------------
procedure TTreeForm.ActionMovePrevExecute(Sender: TObject);
begin
  TreeView.MoveTo(TreeView.FocusedNode, TreeView.FocusedNode.PrevSibling, amInsertBefore, False);

  Modified := True;
end;

procedure TTreeForm.ActionMovePrevUpdate(Sender: TObject);
begin
  ActionMovePrev.Enabled :=
    (TreeView.FocusedNode <> nil) and (TreeView.FocusedNode.PrevSibling <> nil);
end;


//----------------------------------------
// ֈړ
//----------------------------------------
procedure TTreeForm.ActionMoveNextExecute(Sender: TObject);
begin
  TreeView.MoveTo(TreeView.FocusedNode, TreeView.FocusedNode.NextSibling, amInsertAfter, False);

  Modified := True;
end;

procedure TTreeForm.ActionMoveNextUpdate(Sender: TObject);
begin
  ActionMoveNext.Enabled :=
    (TreeView.FocusedNode <> nil) and (TreeView.FocusedNode.NextSibling <> nil);
end;


//----------------------------------------
// ֈړ
//----------------------------------------
procedure TTreeForm.ActionMoveDownExecute(Sender: TObject);
begin
  TreeView.MoveTo(TreeView.FocusedNode, TreeView.FocusedNode.PrevSibling, amAddChildLast, False);

  Modified := True;
end;

procedure TTreeForm.ActionMoveDownUpdate(Sender: TObject);
begin
  ActionMoveDown.Enabled :=
    (TreeView.FocusedNode <> nil) and (TreeView.FocusedNode.PrevSibling <> nil);
end;


//----------------------------------------
// ֈړ
//----------------------------------------
procedure TTreeForm.ActionMoveUpExecute(Sender: TObject);
begin
  TreeView.MoveTo(TreeView.FocusedNode, TreeView.FocusedNode.Parent, amInsertAfter, False);

  Modified := True;
end;

procedure TTreeForm.ActionMoveUpUpdate(Sender: TObject);
begin
  ActionMoveUp.Enabled :=
    (TreeView.FocusedNode <> nil) and (TreeView.GetNodeLevel(TreeView.FocusedNode) <> 0);
end;


//----------------------------------------
// Rs[
//----------------------------------------
procedure TTreeForm.ActionCopyExecute(Sender: TObject);
begin
  TreeView.CopyToClipBoard;
end;

procedure TTreeForm.ActionCopyUpdate(Sender: TObject);
begin
  ActionCopy.Enabled := TreeView.SelectedCount > 0;
end;


//----------------------------------------
// ؂
//----------------------------------------
procedure TTreeForm.ActionCutExecute(Sender: TObject);
begin
  TreeView.CutToClipBoard;
end;

procedure TTreeForm.ActionCutUpdate(Sender: TObject);
begin
  ActionCut.Enabled := TreeView.SelectedCount > 0;
end;


//----------------------------------------
// \t
//----------------------------------------
procedure TTreeForm.ActionPasteExecute(Sender: TObject);
begin
  if True = TreeView.PasteFromClipboard then
  begin
    Modified := True;
    //TreeView.CancelCutOrCopy;
  end;
end;

procedure TTreeForm.ActionPasteUpdate(Sender: TObject);
begin
  ActionPaste.Enabled :=
    (tsCopyPending in TreeView.TreeStates) or (tsCutPending in TreeView.TreeStates);
end;


//----------------------------------------
// qm[hׂĊJ
//----------------------------------------
procedure TTreeForm.ActionExpandAllExecute(Sender: TObject);
var
  Nodes: TNodeArray;
  I: Integer;
begin
  Nodes := TreeView.GetSortedSelection(False);
  for I := 0 to High(Nodes) do
  begin
    TreeView.FullExpand(Nodes[I]);
  end;
end;

procedure TTreeForm.ActionExpandAllUpdate(Sender: TObject);
begin
  ActionExpandAll.Enabled := TreeView.SelectedCount > 0;
end;


procedure TTreeForm.ActionMergeNodeUpdate(Sender: TObject);
begin
end;


//----------------------------------------
// O̕ύX
//----------------------------------------
procedure TTreeForm.ActionChangeCaptionExecute(Sender: TObject);
begin
  TreeView.EditNode(TreeView.FocusedNode, -1);
end;


//----------------------------------------
// \[g̔r֐
//----------------------------------------
procedure TTreeForm.TreeViewCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
begin
  Result := CompareText(NodeData(Node1).Caption, NodeData(Node2).Caption);
end;


//----------------------------------------
// m[h̃f[^TCYKvɂȂƂɌĂяo
//----------------------------------------
procedure TTreeForm.TreeViewGetNodeDataSize(Sender: TBaseVirtualTree;
  var NodeDataSize: Integer);
begin
  NodeDataSize := SizeOf(TNodeData);
end;


//----------------------------------------
// m[h̃eLXgKvɂȂƂɌĂяo
//----------------------------------------
procedure TTreeForm.TreeViewGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
begin
  if TextType = ttNormal then
  begin
    CellText := NodeData(Node).Caption;
  end
  else
  begin
    CellText := '';
  end;
end;


//----------------------------------------
// m[h
//----------------------------------------
procedure TTreeForm.TreeViewInitNode(Sender: TBaseVirtualTree; ParentNode,
  Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
begin
  InitNode(Sender, Node);
end;


//----------------------------------------
// m[hj
//----------------------------------------
procedure TTreeForm.TreeViewFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
begin
  FreeAndNil(NodeData(Node).Strings);
  NodeData(Node).Caption := '';
  MainForm.TreeViewFreeNode(Sender);
end;


//----------------------------------------
// m[h̃eLXgύXۂɌĂяo
//----------------------------------------
procedure TTreeForm.TreeViewNewText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; NewText: WideString);
begin
  if Pos(#13#10, NewText) = 0 then
  begin
    NodeData(Node).Caption := NewText;
  end
  else
  begin
    NodeData(Node).Caption := Copy(NewText, 0, Pos(#13#10, NewText)-1);
  end;

  MainForm.TreeViewTextChange(Sender, NodeData(Node).Caption);
end;


//----------------------------------------
// m[h̃eLXgύXɌĂяo
//----------------------------------------
procedure TTreeForm.TreeViewEdited(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
begin
  Modified := True;
end;


//----------------------------------------
// c[ڕύXOCxg
//----------------------------------------
procedure TTreeForm.TreeViewFocusChanging(Sender: TBaseVirtualTree;
  OldNode, NewNode: PVirtualNode; OldColumn, NewColumn: TColumnIndex;
  var Allowed: Boolean);
begin
  // CtH[ɈˑȂA
  // CtH[Ńc[̃CxgLb`̂ʓ|
  // CtH[̃c[ύXOĂ
  MainForm.TreeViewFocusChanging(Sender, OldNode, NewNode)
end;


//----------------------------------------
// c[ڕύXCxg
//----------------------------------------
procedure TTreeForm.TreeViewFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
begin
  // CtH[ɈˑȂA
  // CtH[Ńc[̃CxgLb`̂ʓ|
  // CtH[̃c[ύXĂ
  MainForm.TreeViewFocusChanged(Sender, Node);
end;


//----------------------------------------
// AvP[VƎ̃f[^Xg[ǂݏo
// (OLEhbOhbvOLE\t̍ۂ̓ƎtH[}bg)
//----------------------------------------
procedure TTreeForm.TreeViewLoadNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Stream: TStream);
var
  Data: PNodeData;
  MS: TMemoryStream;
  Len: Integer;
  P: Pointer;
begin
  // ̃f[^@@@
  Data := NodeData(Node);
  Data.StateIndex := -1;
  Data.Strings := TStringList.Create;

  // LvV
  Stream.Read(Len, SizeOf(Len));
  SetLength(Data.Caption, Len);
  Stream.Read(PChar(Data.Caption)^, Len);

  Stream.Read(Data.SelStart, SizeOf(Data.SelStart));
  Stream.Read(Data.SelLength, SizeOf(Data.SelLength));

  // Xe[gC[Wԍ
  Stream.Read(Data.StateIndex, SizeOf(Data.StateIndex));

  // {eLXg
  Stream.Read(Len, SizeOf(Len));
  if Len <> 0 then
  begin
    MS := TMemoryStream.Create;
    try
      MS.Size := Len;
      P := MS.Memory;
      Stream.Read(P^, Len);
      MS.Position := 0;
      Data.Strings.LoadFromStream(MS);
      Data.NodeNo := NodeNo;
      Inc(NodeNo);
    finally
      MS.Free;
    end;
  end;
end;


//----------------------------------------
// AvP[VƎ̃f[^Xg[֕ۑ
// (OLEhbOhbvOLE\t̍ۂ̓ƎtH[}bg)
//----------------------------------------
procedure TTreeForm.TreeViewSaveNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Stream: TStream);
var
  Data: PNodeData;
  MS: TMemoryStream;
  Len: Integer;
  P: Pointer;
begin
  Data := NodeData(Node);

  // LvV
  Len := Length(Data.Caption);
  Stream.Write(Len, SizeOf(Len));
  Stream.Write(PChar(Data.Caption)^, Len);

  Len := SizeOf(Data.SelStart);
  Stream.Write(Data.SelStart, Len);
  Len := SizeOf(Data.SelLength);
  Stream.Write(Data.SelLength, Len);

  // Xe[gC[Wԍ
  Len := SizeOf(Data.StateIndex);
  Stream.Write(Data.StateIndex, Len);

  // {eLXg
  if Len <> 0 then
  begin
    MS := TMemoryStream.Create;
    try
      Data.Strings.SaveToStream(MS);
      Len := MS.Size;
      Stream.Write(Len, SizeOf(Len));
      P := MS.Memory;
      Stream.Write(P^, Len);
    finally
      MS.Free;
    end;
  end;
end;


//----------------------------------------
// Nana`ŕۑ
//----------------------------------------
procedure TTreeForm.SaveToNanaFile(const Path: TFileName);
var
  SL: TStringList;
  Node: PVirtualNode;
  MSIn: TMemoryStream;
  MSOut: TMemoryStream;

  // 葱(ۑpeLXg쐬)
  procedure CreateNodes(SL: TStrings; Node: PVirtualNode);
  var
    Data: PNodeData;
    ChildNode: PVirtualNode;
    Expand: Integer;
    Focus: Integer;
  begin
    while Assigned(Node) do
    begin
      Data := TreeForm.NodeData(Node);

      // ^Cg
      SL.Add(sChapter + Data.Caption);

      // Agr[g
      Expand := 0;
      if TreeView.Expanded[Node.Parent] then Expand := 1;
      Focus := 0;
      if TreeView.FocusedNode = Node then Focus := 1;
      SL.Add(
        sAttr +
        IntToStr(Data.StateIndex) + ',' +   // C[Wԍ
        IntToStr(Expand) + ',' +            // em[h̊J
        IntToStr(Focus));                   // m[h̃tH[JX

      // {
      SL.Add(sContent);
      if Data.Strings.Text <> '' then
      begin
        SL.Add(Copy(Data.Strings.Text, 1, Length(Data.Strings.Text) -2));  // Text̍Ō#13#10邽߁A-2
      end;
      SL.Add(sUncontent);

      ChildNode := Node.FirstChild;
      if Assigned(ChildNode) then
      begin
        SL.Add(sGroup);
        CreateNodes(SL, ChildNode);   // ċA
        SL.Add(sUngroup);
      end;
      Node := Node.NextSibling;
    end;
  end;

begin
  SL := TStringList.Create;
  Screen.Cursor := crHourGlass;
  try
    Node := TreeView.RootNode.FirstChild;
    CreateNodes(SL, Node);

    // Ver.105 kNanat@C(*.nnz)Ή 
    if ExtractFileExt(Path) = '.nnz' then
    begin
      MSIn := TMemoryStream.Create;
      MSOut := TMemoryStream.Create;
      try
        SL.SaveToStream(MSIn);
        CompressStream(MSIn, MSOut);
        MSOut.SaveToFile(Path);
      finally
        MSOut.Free;
        MSIn.Free;
      end;
    end
    else
    begin
      SL.SaveToFile(Path);
    end;
    // Ver.105 kNanat@C(*.nnz)Ή ܂

  finally
    Screen.Cursor := crDefault;
    SL.Free;
  end;
end;


//----------------------------------------
// tH[̏
//----------------------------------------
procedure TTreeForm.FormCreate(Sender: TObject);
begin
  // VirtualStringTreeɃf[^̃TCYZbg(KKv݂)
  TreeView.NodeDataSize := SizeOf(TNodeData);
end;


//----------------------------------------
// \[g
//----------------------------------------
procedure TTreeForm.ActionSortNodeExecute(Sender: TObject);
var
  Dlg: TSortDlg;
  Node: PVirtualNode;
  Recursive: Boolean;
  SD: TSortDirection;
begin
  Dlg := TSortDlg.Create(Self);
  try
    // qm[hꍇ́Aqm[h̃\[gIsɂ
    if TreeView.FocusedNode.ChildCount = 0 then
    begin
      Dlg.RadioButtonChild.Enabled := False;
    end;
    // Zm[hꍇ́AZm[h̑Isɂ
    if TreeView.FocusedNode.Parent.ChildCount = 1 then
    begin
      Dlg.RadioButtonSibling.Enabled := False;
    end;

    if Dlg.RadioButtonChild.Enabled then
    begin
      Dlg.RadioButtonChild.Checked := True;
    end
    else if Dlg.RadioButtonSibling.Enabled then
    begin
      Dlg.RadioButtonSibling.Checked := True;
    end;

    if mrOK = Dlg.ShowModal then
    begin
      // \[gΏۃm[h
      Node := nil;
      if Dlg.RadioButtonChild.Checked then
      begin
        Node := TreeView.FocusedNode;
      end;
      if Dlg.RadioButtonSibling.Checked then
      begin
        Node := TreeView.FocusedNode.Parent;
      end;

      // ċA\[gǂ̐ݒ
      Recursive := False;
      if Dlg.CheckBoxRecursive.Checked then
      begin
        Recursive := True;
      end;

      // \[g@
      SD := sdAscending;
      if Dlg.CheckBoxDescending.Checked then
      begin
        SD := sdDescending;
      end;

      // \[gs
      SortNode(Node, Recursive, SD);
      Modified := True;
    end;
  finally
    Dlg.Release;
  end;
end;

procedure TTreeForm.ActionSortNodeUpdate(Sender: TObject);
begin
  ActionSortNode.Enabled :=
    (TreeView.FocusedNode.ChildCount > 0) or
    (TreeView.FocusedNode.Parent.ChildCount > 1);
end;


//----------------------------------------
// Oֈړ
//----------------------------------------
procedure TTreeForm.ActionPrevFocusExecute(Sender: TObject);
var
  Node: PVirtualNode;
begin
  Node := TreeView.FocusedNode.PrevSibling;
  if Assigned(Node) then
  begin
    TreeView.FocusedNode := Node;
  end;
end;


//----------------------------------------
// ֈړ
//----------------------------------------
procedure TTreeForm.ActionNextFocusExecute(Sender: TObject);
var
  Node: PVirtualNode;
begin
  Node := TreeView.FocusedNode.NextSibling;
  if Assigned(Node) then
  begin
    TreeView.FocusedNode := Node;
  end;
end;


//----------------------------------------
// tH[JX̂m[h̃eLXg𑾎ɂ
//----------------------------------------
procedure TTreeForm.TreeViewPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
begin
  if Node = TreeView.FocusedNode then
  begin
    TargetCanvas.Font.Style := TargetCanvas.Font.Style + [fsBold];
  end;
end;


end.
