unit NanaRichEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, JvExStdCtrls,
  RichEdit, JvRichEdit, Math;

type
  TEditMarks = class;

  TEditMark = class(TPersistent)
  private
    FOwner   : TEditMarks;
    FEnabled : Boolean;
    FColor   : TColor;
    FMark    : string;
    procedure SetEnabled(Value: Boolean);
    procedure SetColor(Value: TColor);
    procedure SetMark(Value: string);
  public
    constructor Create(AOwner: TEditMarks);
  published
    property Enabled : Boolean read FEnabled write SetEnabled;
    property Color   : TColor  read FColor write SetColor default clSilver;
    property Mark    : string read FMark write SetMark;
  end;

  TNanaRichEdit = class;
  TEditMarks = class(TPersistent)
  private
   FOwner: TNanaRichEdit;
   FEofMark  : TEditMark;
   FCrlfMark : TEditMark;
   FLineMark : TEditMark;
   FTabMark  : TEditMark;
   FSpaceMark: TEditMark;
   procedure UpdateMarker;
  public
    constructor Create(AOwner: TNanaRichEdit);
    destructor  Destroy; override;
  published
    property EofMark: TEditMark read FEofMark write FEofMark;
    property CrlfMark: TEditMark read FCrlfMark write FCrlfMark;
    property LineMark: TEditMark read FLineMark write FLineMark;
    property TabMark: TEditMark read FTabMark write FTabMark;
    property SpaceMark: TEditMark read FSpaceMark write FSpaceMark;
  end;

  TPaintEvent = procedure(Sender: TObject; Msg: TWMPaint) of object;

  TNanaRichEdit = class(TJvRichEdit)
  private
    { Private 錾 }
    FOnPaint: TPaintEvent;

    FCanvas    : TCanvas;
    FEditMarks : TEditMarks;
    procedure DrawEditMarker;
    procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
  protected
    { Protected 錾 }
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    function GetRichEditVersion: Integer;
  published
    { Published 錾 }
    property OnPaint: TPaintEvent read FOnPaint write FOnPaint;
    property EditMarks: TEditMarks read FEditMarks write FEditMarks;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TNanaRichEdit]);
end;

constructor TEditMark.Create(AOwner: TEditMarks);
begin
  inherited Create;
  FOwner := AOwner;
  FColor := clSilver;
end;

procedure TEditMark.SetEnabled(Value: Boolean);
begin
  if Value = FEnabled then Exit;
  FEnabled := Value;
  FOwner.UpdateMarker;
end;

procedure TEditMark.SetColor(Value: TColor);
begin
  if Value = FColor then Exit;
  FColor := Value;
  FOwner.UpdateMarker;
end;

procedure TEditMark.SetMark(Value: string);
begin
  if Value = FMark then Exit;
  FMark := Value;
  FOwner.UpdateMarker;
end;

constructor TEditMarks.Create(AOwner: TNanaRichEdit);
begin
  inherited Create;
  FEofMark   := TEditMark.Create(Self);
  FCrlfMark  := TEditMark.Create(Self);
  FLineMark  := TEditMark.Create(Self);
  FTabMark   := TEditMark.Create(Self);
  FSpaceMark := TEditMark.Create(Self);
  FOwner := AOwner;
  FEofMark.Mark   := '[EOF]';
  FCrlfMark.Mark  := '';
  FLineMark.Mark  := '';
  FTabMark.Mark   := '>';
  FSpaceMark.Mark := '';
end;

destructor  TEditMarks.Destroy;
begin
  FEofMark.Free;
  FCrlfMark.Free;
  FLineMark.Free;
  FTabMark.Free;
  FSpaceMark.Free;
  inherited Destroy;
end;

procedure TEditMarks.UpdateMarker;
begin
  FOwner.Invalidate;
end;

constructor TNanaRichEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCanvas := TCanvas.Create;
  FEditMarks := TEditMarks.Create(Self);
end;

procedure TNanaRichEdit.CreateWnd;
begin
  inherited CreateWnd;
  FCanvas.Handle := GetDC(Handle);
  FCanvas.Brush.Color := clNone;
  FCanvas.Brush.Style := bsClear;
  FCanvas.Pen.Mode := pmCopy;
  FCanvas.Pen.Width := 1;
  FCanvas.Font.Size := 9;
  FCanvas.Font.Name := 'lr SVbN';
end;

procedure TNanaRichEdit.DestroyWnd;
begin
  ReleaseDC(Handle, FCanvas.Handle);
  inherited DestroyWnd;
end;

destructor TNanaRichEdit.Destroy;
begin
  FEditMarks.Free;
  FCanvas.Free;
  inherited Destroy;
end;

function TNanaRichEdit.GetRichEditVersion;
begin
  Result := RichEditVersion;
end;

procedure TNanaRichEdit.DrawEditMarker;
var
  R: TRect;
  pt : TPoint;
  TxtHeight: Integer;
  TxtWidth: Integer;
  DrawRect: TRect;
  StartPos: Integer;
  EndPos: Integer;
  EndPt: TPoint;
  ProgressPos: Integer;
  CharPt: TPoint;
begin
  if (FEditMarks.EofMark.Enabled = False) and
     (FEditMarks.TabMark.Enabled = False) and
     (FEditMarks.SpaceMark.Enabled = False) and
     (FEditMarks.CrlfMark.Enabled = False) and
     (FEditMarks.LineMark.Enabled = False) then Exit;

  SendMessage(Handle, EM_GETRECT, 0, LongInt(@R));

  // EOF̕\
  if (FEditMarks.EofMark.Enabled) then
  begin
    FCanvas.Font.Color := FEditMarks.EofMark.Color;
    pt := Self.GetCharPos(Length(Text));
    TxtHeight := FCanvas.TextHeight('@' + FEditMarks.FEofMark.FMark);
    TxtWidth := FCanvas.TextWidth('@' + FEditMarks.FEofMark.FMark);

    if PtInRect(R, pt) then
    begin
      DrawRect := Rect(pt.X, pt.Y, Min(pt.X + TxtWidth, R.Right), Min(pt.Y + TxtHeight, R.Bottom));
      DrawText(
        FCanvas.Handle,
        PChar('@' + FEditMarks.FEofMark.FMark),
        Length('@' + FEditMarks.FEofMark.FMark),
        DrawRect,
        0);
    end;
  end;

  StartPos := Perform(EM_LINEINDEX, Perform(EM_GETFIRSTVISIBLELINE, 0, 0), 0);
  EndPt := Point(Self.ClientWidth, Self.ClientHeight);
  EndPos := Self.Perform(Messages.EM_CHARFROMPOS, 0, Integer(@EndPt));

  // TAB}[N̕\
  if FEditMarks.TabMark.Enabled then
  begin
    ProgressPos := StartPos;
    FCanvas.Font.Color := FEditMarks.TabMark.Color;
    TxtHeight := FCanvas.TextHeight(FEditMarks.FTabMark.FMark);
    TxtWidth := FCanvas.TextWidth(FEditMarks.FTabMark.FMark);

    while True do
    begin
      ProgressPos := FindText(#9, ProgressPos, EndPos - ProgressPos +1, []);
      if ProgressPos = -1 then Break;

      CharPt := Self.GetCharPos(ProgressPos);
      if PtInRect(R, CharPt) then
      begin
//        FCanvas.TextOut(CharPt.X, CharPt.Y +2, FEditMarks.TabMark.Mark);
        DrawRect := Rect(CharPt.X, CharPt.Y, Min(CharPt.X + TxtWidth, R.Right), Min(CharPt.Y + TxtHeight, R.Bottom));
        DrawText(
          FCanvas.Handle,
          PChar(FEditMarks.FTabMark.FMark),
          Length(FEditMarks.FTabMark.FMark),
          DrawRect,
          0);
      end;
      ProgressPos := ProgressPos + 1;
    end;
  end;

  // Enter}[N̕\
  if FEditMarks.FCrlfMark.Enabled then
  begin
    ProgressPos := StartPos;
    FCanvas.Font.Color := FEditMarks.CrlfMark.Color;
    TxtHeight := FCanvas.TextHeight(FEditMarks.FCrlfMark.FMark);
    TxtWidth := FCanvas.TextWidth(FEditMarks.FCrlfMark.FMark);

    while True do
    begin
      ProgressPos := FindText(#13, ProgressPos, EndPos - ProgressPos +1, []);
      if ProgressPos = -1 then Break;

      CharPt := Self.GetCharPos(ProgressPos);
      if PtInRect(R, CharPt) then
      begin
//        FCanvas.TextOut(CharPt.X, CharPt.Y +2, FEditMarks.CrlfMark.Mark);
        DrawRect := Rect(CharPt.X, CharPt.Y, Min(CharPt.X + TxtWidth, R.Right), Min(CharPt.Y + TxtHeight, R.Bottom));
        DrawText(
          FCanvas.Handle,
          PChar(FEditMarks.FCrlfMark.FMark),
          Length(FEditMarks.FCrlfMark.FMark),
          DrawRect,
          0);
      end;
      ProgressPos := ProgressPos + 1;
    end;
  end;

  // Line}[N(Shift+Enter)̕\
  if FEditMarks.FLineMark.Enabled then
  begin
    ProgressPos := StartPos;
    FCanvas.Font.Color := FEditMarks.LineMark.Color;
    TxtHeight := FCanvas.TextHeight(FEditMarks.FLineMark.FMark);
    TxtWidth := FCanvas.TextWidth(FEditMarks.FLineMark.FMark);

    while True do
    begin
      ProgressPos := FindText(#11, ProgressPos, EndPos - ProgressPos +1, []);
      if ProgressPos = -1 then Break;

      CharPt := Self.GetCharPos(ProgressPos);
      if PtInRect(R, CharPt) then
      begin
//        FCanvas.TextOut(CharPt.X, CharPt.Y +2, FEditMarks.LineMark.Mark);
        DrawRect := Rect(CharPt.X, CharPt.Y, Min(CharPt.X + TxtWidth, R.Right), Min(CharPt.Y + TxtHeight, R.Bottom));
        DrawText(
          FCanvas.Handle,
          PChar(FEditMarks.FLineMark.FMark),
          Length(FEditMarks.FLineMark.FMark),
          DrawRect,
          0);
      end;
      ProgressPos := ProgressPos + 1;
    end;
  end;

  // SpXy[X̕\
  if FEditMarks.FSpaceMark.Enabled then
  begin
    ProgressPos := StartPos;
    FCanvas.Font.Color := FEditMarks.SpaceMark.Color;
    TxtHeight := FCanvas.TextHeight(FEditMarks.FSpaceMark.FMark);
    TxtWidth := FCanvas.TextWidth(FEditMarks.FSpaceMark.FMark);

    while True do
    begin
      ProgressPos := FindText(#$81#$40, ProgressPos, EndPos - ProgressPos +1, [stMatchCase]);
      if ProgressPos = -1 then Break;

      Charpt := Self.GetCharPos(ProgressPos);
      if PtInRect(R, CharPt) then
      begin
//        FCanvas.TextOut(CharPt.X, CharPt.Y +2, FEditMarks.SpaceMark.Mark);
        DrawRect := Rect(CharPt.X, CharPt.Y, Min(CharPt.X + TxtWidth, R.Right), Min(CharPt.Y + TxtHeight, R.Bottom));
        DrawText(
          FCanvas.Handle,
          PChar(FEditMarks.FSpaceMark.FMark),
          Length(FEditMarks.FSpaceMark.FMark),
          DrawRect,
          0);
      end;
      ProgressPos := ProgressPos + 1;
    end;
  end;

end;

procedure TNanaRichEdit.WMPaint(var Msg: TWMPaint);
begin
  inherited;
  DrawEditMarker;
  if Assigned(FOnPaint) then FOnPaint(Self, Msg);
end;

end.
